@@ -30,9 +30,15 @@ sub startup {
     $r->post('/edge')->to('scenario#add_edge');
 
     $r->get('/node/:node_id')->to('node#to_json');
+    # Questa put dovrebbe essere una post
+    $r->put('/node/:node_id')->to('node#update_position');
     $r->post('/node')->to('scenario#add_node');
 
-    $r->get('/scenario')->to('scenario#to_json');
+    $r->get('/scenario/:scenario_id')->to('scenario#to_json');
+    $r->get('/scenario/:scenario_id/task')->to('scenario#run_task');
+
+    $r->get('/slot/:slot_id/data')->to('slot#get_data');
+    $r->post('/slot/:slot_id/data')->to('slot#set_data');
 
 }
 
