@@ -9,6 +9,12 @@ var $j = jQuery.noConflict();
 
 PNI = {
 
+    // Global map id <--> object.
+    // TODO si potrebbe anche togliere, sarebbe meglio.
+    // in particolare l' edge deve essere creato da zero, e il semiEdge deve
+    // essere una classe a parte.
+    elemById: {},
+
     extend: function(subClass, superClass){
 
         for (var key in superClass.prototype) {
@@ -24,16 +30,17 @@ PNI = {
     Theme: {
 
         fontFamily: "Calibri",
-        fontSize: 20,
+        fontSize: 12,
 
         edgeStrokeStyle: "blue",
-        edgeLineWidth: 4,
+        edgeLineWidth: 2,
 
         nodeBoxTransitionDuration: 0.2,
         nodeFill: "orange",
+        nodeHeight: 30,
 
         slotSide: 10,
-        strokeWidth: 4,
+        strokeWidth: 2,
         textFill: "black",
 
     }
@@ -46,11 +53,14 @@ PNI = {
 
 PNI.Elem = function(arg) { 
 
-    this.id = arg.id;
-    this.layer = arg.layer;
+    this.model = arg.model;
     this.scenario = arg.scenario;
-    this.stage = arg.stage;
-    this.uri;
+
+    // TODO aggiustare bene sta cosa dell id, soprattuto per l' edge visto che
+    // glielo aggiungo dopo
+    if( ( typeof this.model != "undefined" ) && ( typeof this.model.id != "undefined" ) ) {
+        PNI.elemById[this.model.id] = this;
+    }
 
 }
 
@@ -60,22 +70,25 @@ PNI.Elem.prototype = { };
 
 // PNI.Scenario //[
 
-PNI.Scenario = function(arg) { 
+PNI.Scenario = function(arg) { //[
+
+    // Set default model.
+    if( typeof arg.model == "undefined" ) {
+        arg.model = {
+            id: 1,
+            nodes: [],
+            edges: []
+        }
+    }
 
     PNI.Elem.call(this,arg);
 
-    this.uri = '/scenario/'+this.id;
-
-    // TODO vedi se esiste lo scenario e renderizzalo, altrimenti chiami
-    // scenario/create
     this.$container = $j("#scenario");
 
     var height = this.$container.height();
     var width = this.$container.width();
 
-    this.inEditor = new PNI.InEditor();
-
-    this.nodeSelector = new PNI.NodeSelector();
+    this.nodeSelector = new PNI.NodeSelector({scenario:this});
     this.nodeSelector.update();
 
     this.edges = [];
@@ -88,53 +101,89 @@ PNI.Scenario = function(arg) {
     this.currenNode = undefined;
     this.semiEdge = undefined;
 
-    // Render elems that are already created.
     this.updateView();
 
     var that = this;
 
-    //this.$container.click( function(e) { });
-
-    this.$container.dblclick( function(e) {
+    this.$container
+    .dblclick( function(evt) {
 
         var nodeSelector = that.nodeSelector;
 
         // Doubleclick toggles nodeSelector visibility.
-        if( nodeSelector.visible ) {
+        if( nodeSelector.isVisible ) {
             nodeSelector.hide();
             return;
         }
 
         var $offset = that.$container.offset();
-        var x = e.pageX - $offset.left; 
-        var y = e.pageY - $offset.top;
+        var x = evt.pageX - $offset.left; 
+        var y = evt.pageY - $offset.top;
 
-        nodeSelector.show(x,y);
+        // Update nodeSelector position and show it.
+        nodeSelector.x = x;
+        nodeSelector.y = y;
+        nodeSelector.show();
 
         nodeSelector.$input.autocomplete({ 
 
-            select: function(ev,ui) {
+            select: function( evt, ui ) {
 
                 nodeSelector.hide();
 
                 that.addNode({
-                    scenario_id: that.id,
                     type: ui.item.value,
                     x: x,
                     y: y
                 });
+            }
+        });
+    })
+    .keypress( function( evt ) {
+
+        if( evt.charCode == 13 ) {
+
+            if ( that.nodeSelector.isVisible === false ) {
+                that.runTask();
+                that.updateView();
+            }
+
+        }
+
+        if( evt.charCode == 36 ) {
+
+            if ( that.nodeSelector.isVisible === true ) {
+
+                // TODO fai che passi tutto in un unico parametro request
+                //var insJSON = JSON.stringify([{label:"in",data:that.nodeSelector.$input.val()}]);
+
+                that.addNode({
+                    type:"Perldata::Scalar",
+                    x:that.nodeSelector.x,
+                    y:that.nodeSelector.y
+                });
+
+                that.nodeSelector.hide();
 
             }
 
-        });
-
+        }
+        if( evt.charCode == 37 ) {
+            console.log("TODO: create hash (dove si trova il mouse)");
+        }
+        if( evt.charCode == 64 ) {
+            console.log("TODO: create array (dove si trova il mouse)");
+        }
     });
-}
+} //]
 
 PNI.Scenario.prototype = {
 
+    // TODO dovrebbe chiamarsi addNodeToModel ??
     addNode: function(requestData) { //[
 
+        requestData.scenario_id = this.model.id;
+
         var that = this;
 
         $j.ajax({
@@ -142,11 +191,18 @@ PNI.Scenario.prototype = {
             url: '/node',
             data: requestData,
             success: function(responseData) {
-                arg = responseData;
-                arg.stage = that.stage;
-                arg.layer = that.layer;
+
+                var arg = {};
+                arg.model = responseData;
                 arg.scenario = that;
-                var node = new PNI.Node(arg);
+                var node;
+
+                if( arg.model.type == "Perldata::Scalar" ) {
+                    node = new PNI.Node.Perldata.Scalar(arg);
+                }
+                else {
+                    node = new PNI.Node(arg);
+                }
 
                 that.nodes.push(node);
             }
@@ -154,57 +210,90 @@ PNI.Scenario.prototype = {
 
     }, //]
 
-    addEdge: function(requestData) { //[
+    //getModel: function() { },
 
-        var that = this;
-
-        $j.ajax({
-            type: 'POST',
-            url: '/edge',
-            data: requestData,
-            success: function(responseData) {
-                arg = responseData;
-                arg.scenario = that;
-                var edge = new PNI.Edge(arg);
-                that.edges.push(edge);
-                that.stage.add(edge.layer);
-            }
-        });
-
-    }, //]
-
-    updateView: function() {
-
-        var controllerRoute = '/scenario';
+    runTask: function() {
 
         var requestData = {};
 
         var that = this;
+        //TODO uniformala scrivendola come 'GET'
+        // ma bisogna fare una GET ?
+        $j.getJSON( 
+            '/scenario/'+that.model.id+'/task', 
+            requestData, function(responseData) {
+                that.model = responseData;
+                that.renderView();
+            });
 
-        $j.getJSON(controllerRoute,requestData, function(responseData){
+    },
 
-            for ( var i in responseData.nodes ) {
+    updateView: function() { //[
 
-                var arg = responseData.nodes[i];
-                arg.scenario = that;
-                arg.stage = that.stage;
-                arg.layer = that.layer;
-                var node = new PNI.Node(arg);
 
-            } 
 
-            for ( var i in responseData.edges ) {
+        // TODO per ora devo resettare nodes e edges
+        this.nodes=[];
+        this.edges=[];
 
-                var arg = responseData.edges[i];
-                arg.scenario = that;
-                arg.stage = that.stage;
-                arg.layer = that.layer;
-                var edge = new PNI.Edge(arg);
+        var requestData = {};
 
-            } 
+        var controllerRoute = '/scenario/'+this.model.id;
 
+        var that = this;
+
+        /*
+        * TODO uniformala alle altre, dovrebbe essere qualcosa del genere
+        $j.ajax(
+        type: 'GET',
+        url: '/scenario',
+        success: function(responseData) 
+        */
+
+        $j.getJSON( controllerRoute, requestData, function(responseData) {
+
+            that.model = responseData;
+            that.renderView();
         });
+    }, //]
 
+    renderView: function() {
+
+        for ( var i in this.model.nodes ) {
+
+            var model = this.model.nodes[i];
+            // TODO dovrebbe funzionare prendere il nodo da PNI.elemById
+            var node;
+
+            var arg = {};
+            arg.scenario = this;
+            arg.model = model;
+
+            if( model.type == "Perldata::Scalar" ) {
+                node = new PNI.Node.Perldata.Scalar(arg);
+            }
+            else {
+                node = new PNI.Node(arg);
+            }
+            this.nodes.push(node);
+        }
+
+        for ( var i in this.model.edges ) {
+
+            var model = this.model.edges[i];
+            // TOD var edge = PNI.elemById[model.id];
+
+            var arg = {};
+            arg.source = PNI.elemById[model.source_id];
+            arg.target = PNI.elemById[model.target_id];
+            arg.scenario = this;
+            arg.model = model;
+
+            var edge = new PNI.Edge(arg);
+            edge.drawLine();
+
+            this.edges.push(edge);
+        } 
     }
 
 };
@@ -213,38 +302,22 @@ PNI.extend(PNI.Scenario,PNI.Elem);
 
 //]
 
-// PNI.InEditor //[
-
-PNI.InEditor = function(arg) {
-
-    this.$container = $j("#in-editor");
-
-}
-
-PNI.InEditor.prototype = {
-
-    hide: function() { 
-
-    },
-
-    show: function(x,y) { 
-
-    }
-
-}; 
-
-//]
-
 // PNI.NodeSelector //[
 
 PNI.NodeSelector = function(arg) {
 
+    this.x = undefined;
+    this.y = undefined;
+    this.scenario = arg.scenario;
+
     this.$container = $j("#node-selector");
     this.$container.hide();
 
     this.$input = $j("#node-selector-input");
 
-    this.visible = false;
+    var that = this;
+
+    this.isVisible = false;
 
 }
 
@@ -252,7 +325,7 @@ PNI.NodeSelector.prototype = {
 
     hide: function() { 
 
-        this.visible = false;
+        this.isVisible = false;
 
         this.$input.autocomplete("close");
         this.$input.val("");
@@ -261,14 +334,14 @@ PNI.NodeSelector.prototype = {
 
     },
 
-    show: function(x,y) {
+    show: function() {
 
-        this.visible = true;
+        this.isVisible = true;
 
         this.$input.val("");
 
         this.$container
-        .css({left:x,top:y})
+        .css({left:this.x,top:this.y})
         .show()
         ;
 
@@ -296,19 +369,22 @@ PNI.NodeSelector.prototype = {
 
 PNI.Node = function(arg) {
 
-    PNI.Elem.call(this,arg);
+    PNI.Elem.call( this, arg );
 
-    this.uri = '/node/'+this.id;
-
-    this.height = 40;
-    this.width = Math.max( arg.label.length * PNI.Theme.fontSize, 3 * PNI.Theme.slotSide );
+    this.layer = new Kinetic.Layer();
+    this.label = this.model.label || 'Node';
+    this.height = PNI.Theme.nodeHeight;
+    this.width = Math.max( this.model.label.length * PNI.Theme.fontSize, 3 * PNI.Theme.slotSide );
 
     // Force x and y to be numbers since they come from JSON data.
-    this.x = Number(arg.x);
-    this.y = Number(arg.y);
+    // TODO in teoria ora dovrebbe essere ok lato server
+    //this.x = Number(arg.x);
+    //this.y = Number(arg.y);
+    this.x = this.model.x;
+    this.y = this.model.y;
 
-    this.ins = arg.ins || [];
-    this.outs = arg.outs || [];
+    this.ins = [];
+    this.outs = [];
 
     this.group = new Kinetic.Group({draggable:true});
 
@@ -336,7 +412,7 @@ PNI.Node = function(arg) {
 
             x: this.x + this.width / 2,
             y: this.y,
-            text: arg.label,
+            text: this.model.label,
             fontSize: PNI.Theme.fontSize,
             fontFamily: PNI.Theme.fontFamily,
             textFill: PNI.Theme.textFill,
@@ -352,11 +428,10 @@ PNI.Node = function(arg) {
 
     // Create in slots //[
 
-        for ( var i in this.ins ) {
+        for ( var i in this.model.ins ) {
 
-            // TODO order si potrebbe ricavare dal node, ins e id dello slot
             var arg = {};
-            arg.id = this.ins[i];
+            arg.model = this.model.ins[i];
             arg.stage = this.stage;
             arg.layer = this.layer;
             arg.node = this;
@@ -364,8 +439,7 @@ PNI.Node = function(arg) {
             arg.scenario = this.scenario;
 
             var slot = new PNI.In(arg);
-            // Overwrite id with object.
-            this.ins[i] = slot;
+            this.ins.push(slot);
             this.group.add(slot.box);
 
         }
@@ -374,10 +448,10 @@ PNI.Node = function(arg) {
 
     // Create out slots //[
 
-        for ( var i in this.outs ) {
+        for ( var i in this.model.outs ) {
 
             var arg = {};
-            arg.id = this.outs[i];
+            arg.model = this.model.outs[i];
             arg.stage = this.stage;
             arg.layer = this.layer;
             arg.node = this;
@@ -385,8 +459,7 @@ PNI.Node = function(arg) {
             arg.scenario = this.scenario;
 
             var slot = new PNI.Out(arg);
-            // Overwrite id with object.
-            this.outs[i] = slot;
+            this.outs.push(slot);
             this.group.add(slot.box);
 
         }
@@ -394,13 +467,13 @@ PNI.Node = function(arg) {
     //]
 
     this.layer.add(this.group);
-    this.stage.add(this.layer);
+    this.scenario.stage.add(this.layer);
 
     var that = this;
 
-    this.group.on("mouseover", function() { document.body.style.cursor = "pointer"; });
-    this.group.on("mouseout", function() { document.body.style.cursor = "default"; });
-    this.group.on("dragend dragmove",function(evt) {
+    this.group.on( "mouseover", function(evt) { document.body.style.cursor = "pointer"; });
+    this.group.on( "mouseout", function(evt) { document.body.style.cursor = "default"; });
+    this.group.on( "dragmove", function(evt) {
 
         // Update node position.
         var position = that.box.getAbsolutePosition();
@@ -415,18 +488,120 @@ PNI.Node = function(arg) {
                 that.outs[i].edges[j].drawLine();
 
             }
-
         }
+    });
+    this.group.on( "dragend",function(evt) {
+
+        var requestData = {x:that.x,y:that.y};
+
+        $j.ajax({
+            type: 'PUT',
+            url: '/node/'+that.model.id,
+            data: requestData
+        });
 
     });
 }
 
-PNI.Node.prototype = {};
+PNI.Node.prototype = { };
 
 PNI.extend(PNI.Node,PNI.Elem);
 
 //]
 
+PNI.Node.Perldata = {};
+
+PNI.Node.Perldata.Scalar = function(arg) {
+
+    PNI.Node.call( this, arg );
+
+    this.$dataEditor = $j('<div class="ui-widget"></div>')
+    .css({top:this.y,left:this.x+this.height})
+    .css({height:this.height,width:10})
+    .css({position:"absolute",display:"block"})
+    .appendTo(this.scenario.$container)
+    .show()
+    ;
+
+    this.$dataEditorInput = $j('<input>')
+    .attr( 'id', this.model.id )
+    .appendTo(this.$dataEditor)
+    .val(this.model.ins[0].data)
+    .show()
+    .focus()
+    ;
+
+    var that = this;
+
+    this.$dataEditorInput
+    .click( function(evt) { that.$dataEditorInput.focus(); } )
+    .focusout( function(evt) {
+
+        // Update model if input data is changed.
+        if( that.model.ins[0].data != that.$dataEditorInput.val() ) {
+
+            that.model.ins[0].data = that.$dataEditorInput.val();
+            var requestData = { data: that.model.ins[0].data };
+
+            // TODO dovrebbe essere un metodo gestito da PNI.Slot
+            $j.ajax({
+                type: 'POST',
+                url: '/slot/'+that.model.ins[0].id+'/data',
+                data: requestData,
+                success: function(responseData) {}
+            });
+
+        } 
+    })
+    ;
+
+    this.group.on( "dragmove", function(evt) {
+
+        // TODO questo e' preso dalla classe padre, CERCA DI FARE QUALCOSA PIU FURBO,
+        // tipo usare il prototype
+
+        // Update node position.
+        var position = that.box.getAbsolutePosition();
+        that.y = position.y;
+        that.x = position.x;
+
+        that.$dataEditor.css({top:that.y,left:that.x+that.height});
+
+        // Draw out edges.
+        for ( var i in that.outs ) {
+
+            for (  var j in that.outs[i].edges ) {
+
+                that.outs[i].edges[j].drawLine();
+
+            }
+        }
+
+    });
+
+    /*
+    var label;
+    var inId = arg.ins[0];
+
+    $j.ajax({
+    type: 'GET',
+    url: '/slot/'+inId+'/data',
+    success: function(responseData) {
+
+    // TODO BRUTTISSIMA SOLUZIONE ma basta che funzioni per ora
+    label = responseData;
+    arg.label = label;
+    PNI.Node.call( this, arg );
+    }
+    });
+    */
+
+}
+
+PNI.Node.Perldata.Scalar.prototype = {};
+
+PNI.extend(PNI.Node.Perldata.Scalar,PNI.Node);
+
 // PNI.Slot //[
 
 PNI.Slot = function(arg) {
@@ -435,8 +610,7 @@ PNI.Slot = function(arg) {
 
     this.node = arg.node;
     this.order = arg.order;
-
-    this.uri = '/scenario/'+this.scenario.id+'/node/'+this.node.id;
+    this.data = arg.data;
 
     this.box = new Kinetic.Rect({ 
 
@@ -452,13 +626,13 @@ PNI.Slot = function(arg) {
 
     var that = this;
 
-    this.box.on("mouseover", function() {
+    this.box.on("mouseover", function(evt) {
 
         that.scenario.currenSlot = that;
 
     });
 
-    this.box.on("mouseout", function() {
+    this.box.on("mouseout", function(evt) {
 
         that.scenario.currenSlot = undefined;
 
@@ -468,12 +642,29 @@ PNI.Slot = function(arg) {
 
 PNI.Slot.prototype = { 
 
+    getData: function() { 
+
+        var that = this;
+
+        $j.ajax({
+            type: 'GET',
+            url: '/slot/'+that.model.id+'/data',
+            success: function(responseData) {
+
+                that.data = responseData;
+
+            }
+        });
+
+        return this.data;
+    },
+
     getX: function() {
 
         var x = this.node.x;
 
         if( this.order > 0 ) {
-            x += this.order * ( this.node.width - PNI.Theme.slotSide ) / ( this.node.ins.length - 1 ); 
+            x += this.order * ( this.node.width - PNI.Theme.slotSide ) / ( this.node.model.ins.length - 1 ); 
         }
 
         return x;
@@ -490,46 +681,43 @@ PNI.extend(PNI.Slot,PNI.Elem);
 
 PNI.In = function(arg) {
 
-    PNI.Slot.call(this,arg);
-
-    this.uri = this.uri+'/in/'+this.id;
+    PNI.Slot.call( this, arg );
 
     this.edge;
 
     var that = this;
 
-    this.box.on("click",function() {
+    this.box.on( "click" , function() {
 
         if( typeof that.scenario.semiEdge != "undefined" ) {
 
-            // Turn semiEdge into an edge adding its target.
-            that.edge = that.scenario.semiEdge;
+            // Turn semiEdge into an edge and add its target.
+            var edge = that.scenario.semiEdge;
             that.scenario.semiEdge = undefined;
-            that.edge.target = that;
+            edge.target = that;
+
+            // TODO dovrei usare questo ma non funziona
+            // anche se documentato:
+            // that.scenario.stage.off( "mousemove" );
+            that.scenario.stage.on( "mousemove" , function() {} );
+
 
             var requestData = {
-                scenario_id: that.scenario.id,
-                source_id: that.edge.source.id,
-                target_id: that.edge.target.id
+                scenario_id: edge.scenario.model.id,
+                source_id: edge.source.model.id,
+                target_id: edge.target.model.id
             };
 
-            console.log(requestData);
-            console.log(that);
-
             $j.ajax({
                 type: 'POST',
                 url: '/edge',
                 data: requestData,
                 success: function(responseData) {
-
-                    that.scenario.edges.push(that.edge);
-                    that.stage.add(that.edge.layer);
-
+                    edge.model = responseData; 
+                    PNI.elemById[edge.model.id] = edge;
                 }
             });
-
         }
-
     });
 
 }
@@ -548,21 +736,17 @@ PNI.extend(PNI.In,PNI.Slot);
 
 PNI.Out = function(arg) {
 
-    PNI.Slot.call(this,arg);
-
-    this.uri = this.uri+'/out/'+this.id;
+    PNI.Slot.call( this, arg );
 
     this.edges = [];
 
     var that = this;
 
-    this.box.on("click",function() {
+    this.box.on( "click", function(evt) {
 
         var arg = {};
         arg.scenario = that.scenario;
         arg.source = that;
-        arg.stage = that.stage;
-        arg.layer = new Kinetic.Layer();
         var edge = new PNI.Edge(arg);
         edge.drawLine();
         that.edges.push(edge);
@@ -577,28 +761,26 @@ PNI.Out.prototype = {
 
 };
 
-PNI.extend(PNI.Out,PNI.Slot);
+PNI.extend( PNI.Out, PNI.Slot );
 
 //]
 
 // PNI.Edge //[
 
-PNI.Edge = function(arg) {
+PNI.Edge = function(arg) { //[
 
-    PNI.Elem.call(this,arg);
-
-    this.uri = '/edge/'+this.id;
+    PNI.Elem.call( this, arg );
 
     this.source = arg.source;
     this.target = arg.target;
 
-    this.stage.add(this.layer);
+    this.layer = new Kinetic.Layer();
 
     var that = this;
 
     if( this.target !== "undefined" ) {
 
-        this.stage.on("mousemove",function() {
+        this.scenario.stage.on( "mousemove", function(evt) {
             that.drawLine();
         });
 
@@ -606,11 +788,12 @@ PNI.Edge = function(arg) {
 
     }
 
-}
+    this.scenario.stage.add(this.layer);
+} //]
 
 PNI.Edge.prototype = {
 
-    drawLine: function() {
+    drawLine: function() { //[
 
         var context = this.layer.getContext();
         this.layer.clear();
@@ -622,7 +805,7 @@ PNI.Edge.prototype = {
         var endX = this.getEndX();
 
         context.beginPath();
-        context.moveTo(startX, startY);
+        context.moveTo( startX, startY );
         context.bezierCurveTo(
             startX, 
             (startY+endY)/2, 
@@ -635,7 +818,7 @@ PNI.Edge.prototype = {
         context.lineWidth = PNI.Theme.edgeLineWidth;
         context.stroke();
 
-    },
+    }, //]
 
     getEndY: function() { //[
 
@@ -644,15 +827,14 @@ PNI.Edge.prototype = {
             return this.target.getY() + ( PNI.Theme.slotSide / 2 );
 
         }
-
         else {
 
-            var position = this.stage.getMousePosition();
+            var position = this.scenario.stage.getMousePosition();
             return position.y;
 
         }
 
-    },//]
+    }, //]
 
     getEndX: function() { //[
 
@@ -661,10 +843,9 @@ PNI.Edge.prototype = {
             return this.target.getX() + ( PNI.Theme.slotSide / 2 );
 
         }
-
         else {
 
-            var position = this.stage.getMousePosition();
+            var position = this.scenario.stage.getMousePosition();
             return position.x;
 
         }
@@ -673,7 +854,7 @@ PNI.Edge.prototype = {
 
 };
 
-PNI.extend(PNI.Edge,PNI.Elem);
+PNI.extend( PNI.Edge, PNI.Elem );
 
 //]
 
