@@ -3,6 +3,7 @@ use PNI::Mo;
 extends 'PNI::Node';
 
 use PNI::Edge;
+use PNI::Elem;
 use PNI::Node;
 use PNI::Set;
 
@@ -16,6 +17,7 @@ sub add_edge {
     my $self = shift;
 
     my $edge = PNI::Edge->new(@_);
+
     return $self->edges->add($edge);
 }
 
@@ -28,7 +30,20 @@ sub add_node {
 
     $node_class->require or return;
 
-    my $node = $node_class->new( type => $type, @_ );
+    my $arg = {@_};
+
+    my $node = $node_class->new(
+        type => $type,
+        x    => $arg->{x},
+        y    => $arg->{y},
+    );
+
+    for ( @{ $arg->{ins} } ) {
+        my $data  = $_->{data};
+        my $label = $_->{label};
+
+        $node->in($label)->data($data);
+    }
 
     return $self->nodes->add($node);
 }
@@ -41,6 +56,17 @@ sub add_scenario {
     return $self->scenarios->add($scenario);
 }
 
+sub by_id {
+    my $elem = PNI::Elem::by_id(@_);
+
+    if ( defined $elem and $elem->isa('PNI::Scenario') ) {
+        return $elem;
+    }
+    else {
+        return;
+    }
+}
+
 sub del_edge {
     my $self = shift;
 
@@ -227,6 +253,14 @@ A L<PNI::Set> containing <PNI::Scenario>s.
 
     $sub_scenario = $scenario->add_scenario;
 
+=head2 by_id
+
+    use PNI::Scenario;
+
+    my $scenario = PNI::Scenario::by_id($scenario_id);
+
+Given a scenario  id, returns a reference to the scenario.
+
 =head2 del_edge
 
 =head2 del_node
